@@ -25,7 +25,10 @@ Classes:
 
 """
 
-import cPickle
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 import datetime
 import hashlib
 import os
@@ -38,7 +41,7 @@ class Cache(object):
 
     """A cache that stores its data on the file system.
 
-    It uses cPickle to store objects into cache file. It uses appdirs to ensure
+    It uses pickle to store objects into cache file. It uses appdirs to ensure
     that cache files are stored in platform-appropriate, application-specific
     cache directories. Cached data can optionally expire after a certain amount
     of time.
@@ -56,19 +59,23 @@ class Cache(object):
         '9729825'. Next, it prints the value of 'chicago'.
 
     Attributes:
-        cachename: the cache's name, as passed to the Cache constructor.
-        cachedir: the cache's parent directory, as determined by
+        cachedir: the cache file's parent directory, as determined by
             appdirs.user_cache_dir().
+        cachename: the cache's name, as passed to the Cache constructor.
         filename: the cache's filename. It is formed by passing cachename to
             hashlib's sha1() constructor.
 
     Methods:
-        set: store data in the cache.
+        delete: delete the cache file.
+        flush: clear all data from the cache.
         get: get data from the cache.
         invalidate: force data to expire.
+        items: return a list of the cache's keys and values.
+        keys: return a list of the cache's keys.
         remove: remove data from the cache.
-        flush: clear all data from the cache.
-        delete: delete the cache file.
+        set: store data in the cache.
+        set_default: if valid data exists, get it; if not, store it.
+        values: return a list of the cache's values.
 
     """
 
@@ -91,14 +98,162 @@ class Cache(object):
         """
         if appauthor is None or appauthor == "":
             appauthor = appname
-        self.cachename = cachename
+        self.cachename = cachename.encode('utf-8')
         self.cachedir = appdirs.user_cache_dir(appname, appauthor)
         self.filename = os.path.join(self.cachedir,
-                                     hashlib.sha1(cachename).hexdigest())
-        if os.access(self.filename, os.F_OK) is False:
+                                     hashlib.sha1(self.cachename).hexdigest())
+        if not os.access(self.filename, os.F_OK):
             self._create()
             self.flush()
 
+    def delete(self):
+        """Delete the cache file.
+
+        On Windows, if the file is in use by another application, an exception
+        is raised. See os.remove() for more information.
+
+        Raises:
+            exceptions.OSError: the cache file does not exist.
+
+        """
+        os.remove(self.filename)
+
+    def flush(self):
+        """Clear all data in the cache.
+
+        This removes all key/value pairs from the cache.
+
+        Raises:
+            exceptions.IOError: the cache file does not exist.
+
+        """
+        if not os.access(self.filename, os.F_OK):
+            raise IOError("the following cache file does not exist: %s"
+                          % self.filename)
+        else:
+            self._write({})
+
+    def get(self, key, override=False):
+        """Get data from the cache.
+
+        All data stored under *key* is returned. If the data is expired,
+        None is returned. Expired data is returned if *override* is True.
+
+        Args:
+            key: (string) the name of the data to fetch.
+            override: (bool) return expired data; defaults to False.
+
+        Returns:
+            the requested data or None if the requested data has expired.
+
+        Raises:
+            exceptions.KeyError: *key* was not found.
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+            pickle.UnpicklingError: there was a problem unpickling an object.
+
+        """
+        data = self._read()
+        if not self._is_expired(data[key]) or override:
+            return data[key]["data"]
+        else:
+            return None
+
+    def invalidate(self, key=None):
+        """Force data to expire.
+
+        After forcing *key* to expire, calling get() on *key* will return
+            None.
+
+        If *key* is None, then all data is forced to expire.
+
+        Args:
+            key: (string or None) the name of the data to invalidate; if None,
+                defaults to all data.
+
+        Raises:
+            exceptions.KeyError: *key* was not found.
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+
+        """
+        data = self._read()
+        now = datetime.datetime.now()
+        if key is None:
+            for k in data.keys():
+                data[k]["expires"] = now
+        else:
+            data[key]["expires"] = now
+        self._write(data)
+
+    def items(self, override=False):
+        """Return a list of the cache's keys and values.
+
+        By default, only keys and values of non-expired data are returned.
+        If *override* is True, then all keys and values are returned.
+
+        Args:
+            override: (bool) return expired keys and values; defaults to False.
+        
+        Returns:
+            a list of the cache keys/values, where each pair is a tuple.
+
+        Raises:
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+            pickle.UnpicklingError: there was a problem unpickling an object.
+
+        """
+        data = self._read()
+        result = []
+        for k, v in data.items():
+            if not self._is_expired(v) or override:
+                result.append((k, v["data"]))
+        return result
+
+    def keys(self, override=False):
+        """Return a list of the cache's keys.
+
+        By default, only keys that have valid data are returned. If *override*
+        is True, then all keys are returned.
+
+        Args:
+            override: (bool) return expired data's keys; defaults to False.
+
+        Returns:
+            a list of the cache's keys.
+
+        Raises:
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+            pickle.UnpicklingError: there was a problem unpickling an object.
+
+        """
+        data = self._read()
+        result = []
+        for k in data.keys():
+            if not self._is_expired(data[k]) or override:
+                result.append(k)
+        return result
+
+    def remove(self, key):
+        """Remove data from the cache.
+
+        All data stored under *key* is deleted from the cache.
+
+        Args:
+            key: (string) the name of the data to remove.
+
+        Raises:
+            exceptions.KeyError: *key* was not found.
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+
+        """
+        data = self._read()
+        del data[key]
+        self._write(data)
+
     def set(self, key, value, timeout=None):
         """Store data in the cache.
 
@@ -127,112 +282,100 @@ class Cache(object):
         data[key] = {"expires": expires, "data": value}
         self._write(data)
 
-    def get(self, key, override=False):
-        """Get data from the cache.
+    def set_default(self, key, default=None, timeout=None):
+        """If *key* exists, return its value; if not, create *key*.
 
-        All data stored under *key* is returned. If the data is expired,
-        None is returned. Expired data is returned if *override* is True.
+        If *key* is in the cache and its data is valid, return its value.
+        If not, store *key* with *default* value into the cache for *timeout*
+        seconds and return *default*.
+
+        Works like dict.setdefault().
+
+        Args:
+            key: (string) the name of the data to fetch/store.
+            default: default data to store and return if *key* doesn't exist or
+                doesn't have valid data. Defaults to None.
+            timeout: (int, long, float, or None) how long in seconds *default*
+                should be considered valid; if None, defaults to forever.
+
+        Returns:
+            the value of *key* if it exists and is valid; if not, then the
+                value of *default*.
+
+        Raises:
+            exceptions.IOError: the cache file does not exist or cannot be
+                read.
+            pickle.UnpicklingError: there was a problem unpickling an object.
+            pickle.PicklingError: an unpicklable object was passed.
+
+        """
+        data = self._read()
+        k = data[key] if key in data else None
+        if k is not None and not self._is_expired(k):
+            return k["data"]
+        else:
+            if timeout is None:
+                expires = None
+            else:
+                expires = (datetime.datetime.now() +
+                           datetime.timedelta(seconds=timeout))
+            data[key] = {"expires": expires, "data": default}
+            self._write(data)
+            return default
+
+    def values(self, override=False):
+        """Return a list of the cache's values.
+
+        By default, only values that are not expired are returned. If
+        *override* is True, then all values are returned.
 
         Args:
-            key: (string) the name of the data to fetch.
             override: (bool) return expired data; defaults to False.
 
         Returns:
-            the requested data or None if the requested data has expired.
+            a list of the cache's values.
 
         Raises:
-            exceptions.KeyError: *key* was not found.
             exceptions.IOError: the cache file does not exist or cannot be
                 read.
             pickle.UnpicklingError: there was a problem unpickling an object.
 
         """
         data = self._read()
-        if (data[key]["expires"] is None or
-                ((datetime.datetime.now() -
-                  data[key]["expires"]).total_seconds() < 0) or
-                override is True):
-            return data[key]["data"]
-        else:
-            return None
-
-    def invalidate(self, key):
-        """Force data to expire.
-
-        After forcing *key* to expire, calling get() on *key* will return
-            None.
-
-        Args:
-            key: (string) the name of the data to invalidate.
-
-        Raises:
-            exceptions.KeyError: *key* was not found.
-            exceptions.IOError: the cache file does not exist or cannot be
-                read.
-
-        """
-        data = self._read()
-        data[key]["expires"] = datetime.datetime.now()
-        self._write(data)
-
-    def remove(self, key):
-        """Remove data from the cache.
-
-        All data stored under *key* is deleted from the cache.
-
-        Args:
-            key: (string) the name of the data to remove.
-
-        Raises:
-            exceptions.KeyError: *key* was not found.
-            exceptions.IOError: the cache file does not exist or cannot be
-                read.
-
-        """
-        data = self._read()
-        del data[key]
-        self._write(data)
-
-    def flush(self):
-        """Clear all data in the cache.
-
-        This removes all key/value pairs from the cache.
-
-        Raises:
-            exceptions.IOError: the cache file does not exist.
-
-        """
-        if os.access(self.filename, os.F_OK) is False:
-            raise IOError("the following cache file does not exist: %s"
-                          % self.filename)
-        else:
-            self._write({})
-
-    def delete(self):
-        """Delete the cache file.
-
-        On Windows, if the file is in use by another application, an exception
-        is raised. See os.remove() for more information.
-
-        Raises:
-            exceptions.OSError: the cache file does not exist.
-
-        """
-        os.remove(self.filename)
+        result = []
+        for v in data.values():
+            if not self._is_expired(v) or override:
+                result.append(v["data"])
+        return result
 
     def _create(self):
         """Create a cache file."""
-        if os.access(self.cachedir, os.F_OK) is False:
+        if not os.access(self.cachedir, os.F_OK):
             os.makedirs(self.cachedir)
         f, tmp = tempfile.mkstemp(dir=self.cachedir)
         os.rename(tmp, self.filename)
 
+    def _is_expired(self, data):
+        """Find out if the passed data is expired or not."""
+        if data["expires"] is None:
+            return False
+        elif self._total_seconds((datetime.datetime.now() -
+                                  data["expires"])) < 0:
+            return False
+        else:
+            return True
+
     def _read(self):
-        """Open a file and use cPickle to retrieve its data"""
+        """Open a file and use pickle to retrieve its data"""
         with open(self.filename, "rb") as f:
-            return cPickle.load(f)
+            return pickle.load(f)
+
+    def _total_seconds(self, td):
+        """Calculate the number of seconds in a timedelta object."""
+        return ((td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) /
+                10**6)
 
     def _write(self, data):
-        """Open a file and use cPickle to save data to it"""
+        """Open a file and use pickle to save data to it"""
         with open(self.filename, "wb") as f:
-            cPickle.dump(data, f, cPickle.HIGHEST_PROTOCOL)
+            pickle.dump(data, f, pickle.HIGHEST_PROTOCOL)
